﻿using Hotline.Caching.Interfaces;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Tools;
using Hotline.Snapshot;
using Hotline.Snapshot.Interfaces;
using Hotline.Tools;
using Mapster;
using Novacode.NETCorePort;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Authentications;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Application.Snapshot;
public class OrderSnapshotApplication : IOrderSnapshotApplication, IScopeDependency
{
    private readonly IOrderSnapshotRepository _orderSnapshotRepository;
    private readonly IOrderRepository _orderRepository;
    private readonly ISnapshotOrderPublishRepository _snapshotOrderPublishRepository;
    private readonly ISessionContext _sessionContext;
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;

    public OrderSnapshotApplication(IOrderSnapshotRepository orderSnapshotRepository, IOrderRepository orderRepository, ISnapshotOrderPublishRepository snapshotOrderPublishRepository, ISessionContext sessionContext, ISystemSettingCacheManager systemSettingCacheManager)
    {
        _orderSnapshotRepository = orderSnapshotRepository;
        _orderRepository = orderRepository;
        _snapshotOrderPublishRepository = snapshotOrderPublishRepository;
        _sessionContext = sessionContext;
        _systemSettingCacheManager = systemSettingCacheManager;
    }

    /// <summary>
    /// 添加公开工单
    /// </summary>
    /// <param name="addSnapshotOrderPublishInDto"></param>
    /// <param name="none"></param>
    /// <returns></returns>
    public async Task<string> AddOrderPublishAsync(AddSnapshotOrderPublishInDto dto, CancellationToken cancellation)
    {
        dto.ValidateObject();
        var snapshotOrder = await _orderSnapshotRepository.GetAsync(dto.OrderId)
            ?? throw UserFriendlyException.SameMessage("工单不存在");
        var order = await _orderRepository.Queryable()
            .Where(m => m.Id == dto.OrderId)
            .Select(m => new { m.Id, m.No })
            .FirstAsync(cancellation)
            ?? throw UserFriendlyException.SameMessage("工单不存在");
        var entity = dto.Adapt<SnapshotOrderPublish>();
        entity.Id = order.Id;
        entity.IndustryId = snapshotOrder.IndustryId;
        entity.IndustryName = snapshotOrder.IndustryName;
        entity.No = order.No;
        entity.Status = EOrderSnapshotPublishStatus.Pending;
        return await _snapshotOrderPublishRepository.AddAsync(entity, cancellation);
    }

    /// <summary>
    /// 获取网格员回复集合
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public ISugarQueryable<GuiderReplyItemsOutDto> GetGuiderReplyItemsAsync(GuiderReplyItemsInDto dto)
    {
        var query = _orderSnapshotRepository.Queryable()
            .LeftJoin<Order>((snapshot, order) => snapshot.Id == order.Id)
            .Where((snapshot, order) => snapshot.ReplyResultType != null)
            .WhereIF(dto.No.NotNullOrEmpty(), (snapshot, order) => order.No.Contains(dto.No))
            .WhereIF(dto.Title.NotNullOrEmpty(), (snapshot, order) => order.Title.Contains(dto.Title))
            .WhereIF(dto.GuiderName.NotNullOrEmpty(), (snapshot, order) => snapshot.MemberName.Contains(dto.GuiderName))
            .WhereIF(dto.GuiderPhoneNumber.NotNullOrEmpty(), (snapshot, order) => snapshot.MemberMobile.Contains(dto.GuiderPhoneNumber))
            .WhereIF(dto.NewtorkENumber.NotNullOrEmpty(), (snapshot, order) => snapshot.NetworkENumber.Contains(dto.NewtorkENumber))
            .WhereIF(dto.Satus.HasValue, (snapshot, order) => snapshot.ReplyResultType == dto.Satus)
            .WhereIF(dto.BeginCreationTime.HasValue && dto.EndCreationTime.HasValue, (snapshot, order) => order.CreationTime >= dto.BeginCreationTime && order.CreationTime <= dto.EndCreationTime)
            .Select((snapshot, order) => new GuiderReplyItemsOutDto
            {
                No = order.No,
                Title = order.Title
            }, true);

        return query;
    }

    /// <summary>
    /// 随手拍公开审批集合
    /// </summary>
    /// <returns></returns>
    public ISugarQueryable<GetOrderSnapshotPublishAuditItemsOutDto> GetOrderSnapshotPublishAuditItemsAsync(GetOrderSnapshotPublishAuditItemsInDto dto)
    {
        var query = _snapshotOrderPublishRepository.Queryable()
            .LeftJoin<Order>((publish, order) => publish.OrderId == order.Id)
            .WhereIF(dto.Status == 0, publish => publish.Status == EOrderSnapshotPublishStatus.Pending)
            .WhereIF(dto.Status == 1, publish => publish.Status == EOrderSnapshotPublishStatus.Agree)
            .WhereIF(dto.Status == 2, publish => publish.Status == EOrderSnapshotPublishStatus.Refuse)
            .OrderByDescending((publish, order) => new { publish.Status, publish.CreationTime })
            .Select((publish, order) => new GetOrderSnapshotPublishAuditItemsOutDto
            {
                Id = publish.Id,
                No = order.No,
                Title = order.Title
            }, true);

        return query;
    }

    /// <summary>
    /// 随手拍公开审批详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    public async Task<GetOrderSnapshotPublishAuditDetailOutDto> GetOrderSnapshotPublishAuditDetailAsync(string id)
    {
        var publish = await _snapshotOrderPublishRepository.GetAsync(id);

        var order = await _orderRepository.Queryable()
            .Where(m => m.Id == publish.OrderId)
            .Select<GetOrderSnapshotPublishAuditDetailOutDto>()
            .FirstAsync();

        publish.Adapt(order);
        order.Id = publish.Id;
        return order;
    }

    /// <summary>
    /// 随手拍公开详情
    /// </summary>
    /// <param name="id">工单Id</param>
    /// <returns></returns>
    public async Task<GetOrderSnapshotPublishDetailOutDto> GetOrderSnapshotPublishDetailAsync(string id)
    {
        return await _orderRepository.Queryable()
            .Where(m => m.Id == id)
            .Select<GetOrderSnapshotPublishDetailOutDto>()
            .FirstAsync();
    }

    /// <summary>
    /// 随手拍公开集合
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public ISugarQueryable<GetOrderSnapshotPublishItemsOutDto> GetOrderSnapshotPublishItemsAsync(GetOrderSnapshotPublishItemsInDto dto)
    {
        var query = _orderSnapshotRepository.Queryable(includeDeleted: true)
            .LeftJoin<Order>((snapshot, order) => snapshot.Id == order.Id)
            .LeftJoin<SnapshotOrderPublish>((snapshot, order, publish) => order.Id == publish.OrderId)
            .WhereIF(dto.IndustryId.NotNullOrEmpty(), snapshot => snapshot.IndustryId == dto.IndustryId)
            .WhereIF(dto.AreaCode.NotNullOrEmpty(), (snapshot, order) => order.AreaCode == dto.AreaCode)
            .WhereIF(dto.AcceptTypeCode.NotNullOrEmpty(), (snapshot, order) => order.AcceptTypeCode == dto.AcceptTypeCode)
            .WhereIF(dto.BeginCreationTime.HasValue && dto.EndCreationTime.HasValue, (snapshot, order) => order.CreationTime >= dto.BeginCreationTime && order.CreationTime <= dto.EndCreationTime)
            .WhereIF(dto.Contact.NotNullOrEmpty(), (snapshot, order) => order.Contact.Contains(dto.Contact))
            .WhereIF(dto.FromPhone.NotNullOrEmpty(), (snapshot, order) => order.FromPhone.Contains(dto.FromPhone))
            .WhereIF(dto.IsPublished.HasValue, (snapshot, order, publish) => publish.Status == EOrderSnapshotPublishStatus.Agree)
            .WhereIF(dto.No.NotNullOrEmpty(), (snapshot, order) => order.No.Contains(dto.No))
            .WhereIF(dto.OrderStatus.HasValue, (snapshot, order) => order.Status == dto.OrderStatus)
            .WhereIF(dto.Title.NotNullOrEmpty(), (snapshot, order) => order.Title.Contains(dto.Title))
            //.Where((snapshot, order, publish) => order.Status == EOrderStatus.Filed)
            .OrderByDescending((snapshot, order, publish) => new { publish.Status, snapshot.CreationTime })
            .Select((snapshot, order, publish) =>
            new GetOrderSnapshotPublishItemsOutDto
            {
                PublishStatus = publish.Status
            }, true);

        return query;
    }

    /// <summary>
    /// 获取工单标记集合
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public ISugarQueryable<SignOrderSnapshotItemsOutDto> GetSignOrderSnapshotItemsAsync(SignOrderSnapshotItemsInDto dto)
    {
        var query = _orderSnapshotRepository.Queryable()
            .LeftJoin<Order>((snapshot, order) => snapshot.Id == order.Id)
            .LeftJoin<WorkflowStep>((snapshot, order, step) => step.ExternalId == order.Id)
            .WhereIF(dto.Status == 0, (snapshot, order, step) => step.Tag == TagDefaults.OrderMark) // 待标记
            .WhereIF(dto.Status == 1, (snapshot, order, step) => step.Tag == TagDefaults.OrderMark && step.Status != EWorkflowStepStatus.Handled) // 待标记
            .WhereIF(dto.Status == 2, (snapshot, order, step) => step.Tag == TagDefaults.OrderMark && step.Status == EWorkflowStepStatus.Handled) // 已标记
            .WhereIF(dto.No.NotNullOrEmpty(), (snapshot, order, step) => order.No.Contains(dto.No))
            .WhereIF(dto.Title.NotNullOrEmpty(), (snapshot, order, step) => order.Title.Contains(dto.Title))
            .OrderByDescending((snapshot, order, step) => snapshot.CreationTime)
            .Select((snapshot, order) => new SignOrderSnapshotItemsOutDto
            {
                OrderId = snapshot.Id,
                No = order.No,
                Title = order.Title,
                Content = order.Content,
                FullAddress = order.FullAddress,
                CreationTime = order.CreationTime,
                County = order.County,
                IsSafetyDepartment = snapshot.IsSafetyDepartment,
                NetworkENumber = snapshot.NetworkENumber,
                NetworkRemark = snapshot.NetworkRemark,
                ReplyDate = snapshot.ReplyDate,
                SignTime = snapshot.SignTime,
                SignUserName = snapshot.SignUserName
            });
        return query;
    }

    /// <summary>
    /// 批量设置随手拍公开申请不通过
    /// </summary>
    /// <returns></returns>
    public async Task UpdateOrderSnapshotPublishsStatusRefuseAsync(IList<string> ids)
    {
        await _snapshotOrderPublishRepository.Updateable()
            .SetColumns(m => m.Status, EOrderSnapshotPublishStatus.Refuse)
            .SetColumns(m => m.RefuseRemark, _sessionContext.UserName + "批量设置不公开")
            .SetColumns(m => m.RefuseAuditTime, DateTime.Now)
            .Where(m => ids.Contains(m.Id))
            .ExecuteCommandAsync();
    }

    /// <summary>
    /// 审核随手拍公开申请通过/不通过
    /// </summary>
    /// <returns></returns>
    public async Task UpdateOrderSnapshotPublishStatusAsync(UpdateOrderSnapshotPublishStatusInDto dto)
    {
        var audit = await _snapshotOrderPublishRepository.GetAsync(dto.Id)
            ?? throw UserFriendlyException.SameMessage("审核记录不存在");
        audit.Status = dto.Status;
        audit.AuditTime = DateTime.Now;
        await _snapshotOrderPublishRepository.UpdateAsync(audit);
    }

    /// <summary>
    /// 随手拍工单标记
    /// </summary>
    /// <param name="id"></param>
    /// <param name="snapshotLabels"></param>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public async Task UpdateLabelAsync(string id, IList<Kv>? snapshotLabels)
    {
        if (_systemSettingCacheManager.Snapshot == false) return;

        var isSnapshot = await _orderSnapshotRepository.AnyAsync(m => m.Id == id);
        if (isSnapshot && snapshotLabels == null)
        {
            throw UserFriendlyException.SameMessage("随手拍工单标记不能为空");
        }
        if (isSnapshot == true)
            await _orderSnapshotRepository.UpdateLabelAsync(id, snapshotLabels!);
    }
}
